@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "orzwinmenu.h"
 
@@ -37,7 +37,7 @@ orz_winmenu_get_type (void)
 
   if (!type)
     {
-      static const GTypeInfo info =
+      const static GTypeInfo info =
       {
         sizeof (OrzWinmenuClass),
         NULL,               /* base_init */
@@ -78,6 +78,7 @@ orz_winmenu_init (OrzWinmenu *winmenu)
   winmenu->handler_add          = 0;
   winmenu->handler_change       = 0;
   winmenu->handler_remove       = 0;
+  winmenu->handler_reorder      = 0;
   winmenu->handler_switch       = 0;
   winmenu->handler_destroy_mdi  = 0;
   winmenu->handler_destroy_menu = 0;
@@ -132,17 +133,12 @@ orz_winmenu_add (OrzMdi     *mdi,
         {
           gint i;
 
-          for (i = length - 2; i >= 0; i--)
+          for (i = length - 1; i >= 0; i--)
             if (g_list_nth_data (glist, i)
                                     == (g_list_last (winmenu->glist))->data)
-              {
-                gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
+              break;
+          gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
                                                             menu_item, i + 1);
-                break;
-              }
-          if (i < 0)
-            gtk_menu_shell_append (GTK_MENU_SHELL (winmenu->menu_shell),
-                                                                    menu_item);
         }
       else
         {
@@ -162,8 +158,9 @@ orz_winmenu_add (OrzMdi     *mdi,
                   winmenu->menu_item_foot = gtk_separator_menu_item_new ();
                   gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
                                             winmenu->menu_item_foot, position);
+                  gtk_widget_show (winmenu->menu_item_foot);
                 }
-              menu_item = g_list_nth_data (glist,
+              menu_item_position = g_list_nth_data (glist,
                                     (position < 0 ? length : position) - 1);
               if (position != 0
                         && !GTK_IS_SEPARATOR_MENU_ITEM (menu_item_position)
@@ -172,18 +169,12 @@ orz_winmenu_add (OrzMdi     *mdi,
                   winmenu->menu_item_head = gtk_separator_menu_item_new ();
                   gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
                                             winmenu->menu_item_head, position);
+                  gtk_widget_show (winmenu->menu_item_head);
                   if (position > 0)
                     position++;
                 }
             }
-          if (position < 0)
-            gtk_menu_shell_append (GTK_MENU_SHELL (winmenu->menu_shell),
-                                                                    menu_item);
-          else if (position == 0)
-            gtk_menu_shell_prepend (GTK_MENU_SHELL (winmenu->menu_shell),
-                                                                    menu_item);
-          else
-            gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
+          gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
                                                         menu_item, position);
           if (GTK_IS_MENU (winmenu->menu_shell))
             {
@@ -230,6 +221,66 @@ orz_winmenu_change (OrzMdi     *mdi,
 }
 
 
+static void
+orz_winmenu_reorder (OrzMdi     *mdi,
+                     guint       page_num,
+                     gpointer   *page_data,
+                     OrzWinmenu *winmenu)
+{
+  if (winmenu && winmenu->mdi == mdi && winmenu->menu_shell && winmenu->glist)
+    {
+      gint i;
+      GtkWidget *child, *menu_item;
+
+      child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (mdi), page_num);
+      for (i = g_list_length (winmenu->glist) - 1; i >= 0; i--)
+        {
+          menu_item = g_list_nth_data (winmenu->glist, i);
+          if (g_object_get_data (G_OBJECT (menu_item), "user_data") == child)
+            {
+              g_object_ref (G_OBJECT (menu_item));
+              gtk_container_remove (GTK_CONTAINER (winmenu->menu_shell),
+                                                                    menu_item);
+              winmenu->glist = g_list_remove (winmenu->glist, menu_item);
+              break;
+            }
+        }
+      if (i >= 0)
+        {
+          gint length, position;
+          GList *glist;
+
+          glist = gtk_container_get_children
+                                        (GTK_CONTAINER (winmenu->menu_shell));
+          length = g_list_length (glist);
+          if (page_num == 0)
+            {
+              position = 0 <= winmenu->position
+                        && winmenu->position < length ? winmenu->position : -1;
+              if (winmenu->menu_item_head)
+                for (i = 0; i < length; i++)
+                  if (g_list_nth_data (glist, i) == winmenu->menu_item_head)
+                    {
+                      position = i + 1;
+                      break;
+                    }
+            }
+          else
+            {
+              for (position = length - 1; position >= 0; position--)
+                if (g_list_nth_data (glist, position)
+                                == g_list_nth_data (winmenu->glist, page_num))
+                  break;
+            }
+          g_list_free (glist);
+          gtk_menu_shell_insert (GTK_MENU_SHELL (winmenu->menu_shell),
+                                                        menu_item, position);
+          winmenu->glist = g_list_insert (winmenu->glist, menu_item, page_num);
+        }
+    }
+}
+
+
 static void
 orz_winmenu_remove (OrzMdi     *mdi,
                     guint       page_num,
@@ -280,7 +331,7 @@ orz_winmenu_remove (OrzMdi     *mdi,
                   if (menu_item)
                     gtk_widget_set_sensitive (menu_item, FALSE);
                 }
-              g_free (glist);
+              g_list_free (glist);
             }
         }
     }
@@ -335,6 +386,9 @@ orz_winmenu_dispose (GObject *object)
       if (winmenu->handler_remove)
         g_signal_handler_disconnect (G_OBJECT (winmenu->mdi),
                                                 winmenu->handler_remove);
+      if (winmenu->handler_reorder)
+        g_signal_handler_disconnect (G_OBJECT (winmenu->mdi),
+                                                winmenu->handler_reorder);
       if (winmenu->handler_switch)
         g_signal_handler_disconnect (G_OBJECT (winmenu->mdi),
                                                 winmenu->handler_switch);
@@ -464,6 +518,12 @@ orz_winmenu_set_mdi (OrzWinmenu *winmenu,
                                                 winmenu->handler_remove);
               winmenu->handler_remove = 0;
             }
+          if (winmenu->handler_reorder)
+            {
+              g_signal_handler_disconnect (G_OBJECT (winmenu->mdi),
+                                                winmenu->handler_reorder);
+              winmenu->handler_reorder = 0;
+            }
           if (winmenu->handler_switch)
             {
               g_signal_handler_disconnect (G_OBJECT (winmenu->mdi),
@@ -482,15 +542,17 @@ orz_winmenu_set_mdi (OrzWinmenu *winmenu,
       if (winmenu->mdi)
         {
           winmenu->handler_add = g_signal_connect (G_OBJECT (mdi),
-                    "add-window", G_CALLBACK (orz_winmenu_add), winmenu);
+                "add-window", G_CALLBACK (orz_winmenu_add), winmenu);
           winmenu->handler_change = g_signal_connect (G_OBJECT (mdi),
-                    "change-window", G_CALLBACK (orz_winmenu_change), winmenu);
+                "change-window", G_CALLBACK (orz_winmenu_change), winmenu);
           winmenu->handler_remove = g_signal_connect (G_OBJECT (mdi),
-                    "remove-window", G_CALLBACK (orz_winmenu_remove), winmenu);
+                "remove-window", G_CALLBACK (orz_winmenu_remove), winmenu);
+          winmenu->handler_reorder = g_signal_connect (G_OBJECT (mdi),
+                "reorder-window", G_CALLBACK (orz_winmenu_reorder), winmenu);
           winmenu->handler_switch = g_signal_connect (G_OBJECT (mdi),
-                    "switch-window", G_CALLBACK (orz_winmenu_switch), winmenu);
+                "switch-window", G_CALLBACK (orz_winmenu_switch), winmenu);
           winmenu->handler_destroy_mdi = g_signal_connect (G_OBJECT (mdi),
-                    "destroy", G_CALLBACK (orz_winmenu_destroy_mdi), winmenu);
+                "destroy", G_CALLBACK (orz_winmenu_destroy_mdi), winmenu);
           orz_winmenu_create_menu (winmenu);
         }
     }
