@@ -1,6 +1,6 @@
 /*
     Melody maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include <gdk/gdkkeysyms.h>
 #include "genre.h"
@@ -92,38 +92,41 @@ gchar *def_genre_list[256] = {
 * ja:ジャンルダイアログ関数群                                                 *
 *                                                                             *
 ******************************************************************************/
-GtkListStore *store;
-GtkWidget *tview, *button0, *button1, *button2, *button3, *button4;
+typedef struct _GenreDialog
+{
+  GtkListStore *store;
+  GtkWidget *dialog, *tview, *button0, *button1, *button2, *button3;
+} GenreDialog;
 
 
 /* ja:ボタンの設定 */
 static void
-genre_dialog_sensitive (void)
+genre_dialog_sensitive (GenreDialog *gnrdlg)
 {
   GtkTreePath *path;
   GtkTreeViewColumn *column;
 
-  gtk_tree_view_get_cursor (GTK_TREE_VIEW (tview), &path, &column);
+  gtk_tree_view_get_cursor (GTK_TREE_VIEW (gnrdlg->tview), &path, &column);
   if (path && column)
     {
       gint c, count;
       GList *glist;
 
-      glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (tview));
+      glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (gnrdlg->tview));
       count = g_list_length (glist) - 1;
       g_list_free (glist);
-      c = (gint)g_object_get_data (G_OBJECT (column), "user_data");
-      gtk_widget_set_sensitive (button0, c >= 0 && count < 256);
-      gtk_widget_set_sensitive (button1, c >= 0 && count > 1);
-      gtk_widget_set_sensitive (button2, c > 0);
-      gtk_widget_set_sensitive (button3, c >= 0 && c < count - 1);
+      c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"));
+      gtk_widget_set_sensitive (gnrdlg->button0, c >= 0 && count < 256);
+      gtk_widget_set_sensitive (gnrdlg->button1, c >= 0 && count > 1);
+      gtk_widget_set_sensitive (gnrdlg->button2, c > 0);
+      gtk_widget_set_sensitive (gnrdlg->button3, c >= 0 && c < count - 1);
     }
   else
     {
-      gtk_widget_set_sensitive (button0, FALSE);
-      gtk_widget_set_sensitive (button1, FALSE);
-      gtk_widget_set_sensitive (button2, FALSE);
-      gtk_widget_set_sensitive (button3, FALSE);
+      gtk_widget_set_sensitive (gnrdlg->button0, FALSE);
+      gtk_widget_set_sensitive (gnrdlg->button1, FALSE);
+      gtk_widget_set_sensitive (gnrdlg->button2, FALSE);
+      gtk_widget_set_sensitive (gnrdlg->button3, FALSE);
     }
 }
 
@@ -133,26 +136,27 @@ static void
 genre_dialog_edited (GtkCellRendererText *cell,
                      const gchar         *path_string,
                      const gchar         *text,
-                     gpointer             user_data)
+                     GenreDialog         *gnrdlg)
 {
   gint column;
   GtkTreeIter iter;
   GtkTreePath *path;
 
-  column = (gint)g_object_get_data (G_OBJECT (cell), "user_data");
+  column = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (cell), "user_data"));
   path = gtk_tree_path_new_from_string (path_string);
-  gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
+  gtk_tree_model_get_iter (GTK_TREE_MODEL (gnrdlg->store), &iter, path);
   gtk_tree_path_free (path);
-  gtk_list_store_set (store, &iter,
+  gtk_list_store_set (gnrdlg->store, &iter,
                 column, text && text[0] != '\0' ? g_strdup (text) : NULL, -1);
-  gtk_widget_set_sensitive (button4, TRUE);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (gnrdlg->dialog),
+                                                        GTK_RESPONSE_OK, TRUE);
 }
 
 
 /* ja:新規ボタンが押された */
 static void
-genre_dialog_clicked_new (GtkWidget *widget,
-                          gpointer   user_data)
+genre_dialog_clicked_new (GtkWidget   *widget,
+                          GenreDialog *gnrdlg)
 {
   gchar *title;
   gint i, c, count;
@@ -161,9 +165,9 @@ genre_dialog_clicked_new (GtkWidget *widget,
   GtkTreePath *path;
   GtkTreeViewColumn *column;
 
-  gtk_tree_view_get_cursor (GTK_TREE_VIEW (tview), &path, &column);
-  c = (gint)g_object_get_data (G_OBJECT (column), "user_data");
-  glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (tview));
+  gtk_tree_view_get_cursor (GTK_TREE_VIEW (gnrdlg->tview), &path, &column);
+  c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"));
+  glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (gnrdlg->tview));
   count = g_list_length (glist) - 1;
   g_list_free (glist);
   for (i = 0; i < 256; i++)
@@ -171,45 +175,49 @@ genre_dialog_clicked_new (GtkWidget *widget,
       gint j;
       GtkTreeIter iter;
 
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, i);
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (gnrdlg->store), &iter,
+                                                                    NULL, i);
       for (j = count; j > c; j--)
         {
           gchar *str;
 
-          gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, j - 1, &str, -1);
-          gtk_list_store_set (store, &iter, j, str, -1);
+          gtk_tree_model_get (GTK_TREE_MODEL (gnrdlg->store), &iter,
+                                                            j - 1, &str, -1);
+          gtk_list_store_set (gnrdlg->store, &iter, j, str, -1);
           g_free (str);
         }
     }
   renderer = gtk_cell_renderer_text_new ();
   g_signal_connect (renderer, "edited",
-                                    G_CALLBACK (genre_dialog_edited), store);
-  g_object_set_data (G_OBJECT (renderer), "user_data", (gpointer)count);
+                            G_CALLBACK (genre_dialog_edited), gnrdlg->store);
+  g_object_set_data (G_OBJECT (renderer), "user_data",
+                                                    GINT_TO_POINTER (count));
   title = g_strdup_printf ("%d", count);
   column = gtk_tree_view_column_new_with_attributes (title,
                             renderer, "text", count, "editable", 257, NULL);
   g_free (title);
   gtk_tree_view_column_set_resizable (column, TRUE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tview), column);
-  g_object_set_data (G_OBJECT (column), "user_data", (gpointer)count);
-  gtk_widget_set_sensitive (button4, TRUE);
-  genre_dialog_sensitive ();
+  gtk_tree_view_append_column (GTK_TREE_VIEW (gnrdlg->tview), column);
+  g_object_set_data (G_OBJECT (column), "user_data", GINT_TO_POINTER (count));
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (gnrdlg->dialog),
+                                                        GTK_RESPONSE_OK, TRUE);
+  genre_dialog_sensitive (gnrdlg);
 }
 
 
 /* ja:削除ボタンが押された */
 static void
-genre_dialog_clicked_delete (GtkWidget *widget,
-                             gpointer   user_data)
+genre_dialog_clicked_delete (GtkWidget   *widget,
+                             GenreDialog *gnrdlg)
 {
   gint i, c, count;
   GList *glist;
   GtkTreePath *path;
   GtkTreeViewColumn *column;
 
-  gtk_tree_view_get_cursor (GTK_TREE_VIEW (tview), &path, &column);
-  c = (gint)g_object_get_data (G_OBJECT (column), "user_data");
-  glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (tview));
+  gtk_tree_view_get_cursor (GTK_TREE_VIEW (gnrdlg->tview), &path, &column);
+  c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"));
+  glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (gnrdlg->tview));
   count = g_list_length (glist) - 1;
   g_list_free (glist);
   for (i = 0; i < 256; i++)
@@ -217,156 +225,95 @@ genre_dialog_clicked_delete (GtkWidget *widget,
       gint j;
       GtkTreeIter iter;
 
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, i);
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (gnrdlg->store), &iter,
+                                                                    NULL, i);
       for (j = c; j < count; j++)
         {
           gchar *str;
 
-          gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, j + 1, &str, -1);
-          gtk_list_store_set (store, &iter, j, str, -1);
+          gtk_tree_model_get (GTK_TREE_MODEL (gnrdlg->store), &iter,
+                                                            j + 1, &str, -1);
+          gtk_list_store_set (gnrdlg->store, &iter, j, str, -1);
           g_free (str);
         }
     }
-  gtk_tree_view_remove_column (GTK_TREE_VIEW (tview),
-                    gtk_tree_view_get_column (GTK_TREE_VIEW (tview), count));
-  gtk_widget_set_sensitive (button4, TRUE);
-  genre_dialog_sensitive ();
+  gtk_tree_view_remove_column (GTK_TREE_VIEW (gnrdlg->tview),
+            gtk_tree_view_get_column (GTK_TREE_VIEW (gnrdlg->tview), count));
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (gnrdlg->dialog),
+                                                        GTK_RESPONSE_OK, TRUE);
+  genre_dialog_sensitive (gnrdlg);
 }
 
 
 /* ja:上へボタンが押された */
 static void
-genre_dialog_clicked_up (GtkWidget *widget,
-                         gpointer   user_data)
+genre_dialog_clicked_up (GtkWidget   *widget,
+                         GenreDialog *gnrdlg)
 {
   gint i, c;
   GtkTreePath *path;
   GtkTreeViewColumn *column;
 
-  gtk_tree_view_get_cursor (GTK_TREE_VIEW (tview), &path, &column);
-  c = (gint)g_object_get_data (G_OBJECT (column), "user_data");
+  gtk_tree_view_get_cursor (GTK_TREE_VIEW (gnrdlg->tview), &path, &column);
+  c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"));
   for (i = 0; i < 256; i++)
     {
       gchar *str0, *str1;
       GtkTreeIter iter;
 
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, i);
-      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (gnrdlg->store), &iter,
+                                                                    NULL, i);
+      gtk_tree_model_get (GTK_TREE_MODEL (gnrdlg->store), &iter,
                                         c - 1, &str0, c, &str1, -1);
-      gtk_list_store_set (store, &iter, c - 1,  str1, c,  str0, -1);
+      gtk_list_store_set (gnrdlg->store, &iter, c - 1,  str1, c,  str0, -1);
       g_free (str0);
       g_free (str1);
     }
-  gtk_tree_view_set_cursor (GTK_TREE_VIEW (tview), path,
-            gtk_tree_view_get_column (GTK_TREE_VIEW (tview), c), FALSE);
-  gtk_widget_set_sensitive (button4, TRUE);
-  genre_dialog_sensitive ();
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (gnrdlg->tview), path,
+        gtk_tree_view_get_column (GTK_TREE_VIEW (gnrdlg->tview), c), FALSE);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (gnrdlg->dialog),
+                                                        GTK_RESPONSE_OK, TRUE);
+  genre_dialog_sensitive (gnrdlg);
 }
 
 
 /* ja:下へボタンが押された */
 static void
-genre_dialog_clicked_down (GtkWidget *widget,
-                           gpointer   user_data)
+genre_dialog_clicked_down (GtkWidget   *widget,
+                           GenreDialog *gnrdlg)
 {
   gint i, c;
   GtkTreePath *path;
   GtkTreeViewColumn *column;
 
-  gtk_tree_view_get_cursor (GTK_TREE_VIEW (tview), &path, &column);
-  c = (gint)g_object_get_data (G_OBJECT (column), "user_data");
+  gtk_tree_view_get_cursor (GTK_TREE_VIEW (gnrdlg->tview), &path, &column);
+  c = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "user_data"));
   for (i = 0; i < 256; i++)
     {
       gchar *str0, *str1;
       GtkTreeIter iter;
 
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, i);
-      gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
+      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (gnrdlg->store), &iter,
+                                                                    NULL, i);
+      gtk_tree_model_get (GTK_TREE_MODEL (gnrdlg->store), &iter,
                                         c , &str0, c + 1, &str1, -1);
-      gtk_list_store_set (store, &iter, c ,  str1, c + 1,  str0, -1);
+      gtk_list_store_set (gnrdlg->store, &iter, c ,  str1, c + 1,  str0, -1);
       g_free (str0);
       g_free (str1);
     }
-  gtk_tree_view_set_cursor (GTK_TREE_VIEW (tview), path,
-            gtk_tree_view_get_column (GTK_TREE_VIEW (tview), c + 2), FALSE);
-  gtk_widget_set_sensitive (button4, TRUE);
-  genre_dialog_sensitive ();
-}
-
-
-/* ja:OKボタンが押された */
-static void
-genre_dialog_clicked_ok (GtkWidget *widget,
-                         GtkWidget *dialog)
-{
-  gint i, j, k;
-  GList *glist;
-
-  for (i = 0; i < genres; i++)
-    for (j = 0; j < 256; j++)
-      g_free (genre[i][j]);
-  g_memset (genre, 0, sizeof (gchar *) * genres * 256);
-  glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (tview));
-  genres = g_list_length (glist) - 1;
-  g_list_free (glist);
-  for (i = 0; i < 256; i++)
-    {
-      GtkTreeIter iter;
-
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, i);
-      for (j = 0; j < genres; j++)
-        gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
-                                                        j, &genre[j][i], -1);
-    }
-  for (i = 0; i < genres; i++)
-    {
-      for (j = 0; j < 256; j++)
-        if (genre[i][j])
-          break;
-      if (j >= 256)
-        {
-          genres--;
-          for (j = i; j < genres; j++)
-            for (k = 0; k < 256; k++)
-              genre[j][k] = genre[j + 1][k];
-          for (j = 0; j < 256; j++)
-            genre[genres][j] = NULL;
-        }
-    }
-  gtk_widget_destroy (dialog);
+  gtk_tree_view_set_cursor (GTK_TREE_VIEW (gnrdlg->tview), path,
+    gtk_tree_view_get_column (GTK_TREE_VIEW (gnrdlg->tview), c + 2), FALSE);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (gnrdlg->dialog),
+                                                        GTK_RESPONSE_OK, TRUE);
+  genre_dialog_sensitive (gnrdlg);
 }
 
 
 /* ja:カーソルが移動した */
-#define genre_dialog_cursor_changed genre_dialog_sensitive
-
-
-/* ja:ツリービューの破棄 */
-static void
-genre_dialog_destroy (GtkWidget *widget,
-                      gpointer   user_data)
+void genre_dialog_cursor_changed (GtkTreeView *tview,
+                                  GenreDialog *gnrdlg)
 {
-  gint i, j, count;
-  GList *glist;
-
-  glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (widget));
-  count = g_list_length (glist) - 1;
-  g_list_free (glist);
-  for (i = 0; i < 256; i++)
-    {
-      GtkTreeIter iter;
-
-      gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, i);
-      for (j = 0; j < count; j++)
-        {
-          gchar *str;
-
-          gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, j, &str, -1);
-          gtk_list_store_set (store, &iter, j, NULL, -1);
-          g_free (str);
-        }
-    }
-
+  genre_dialog_sensitive (gnrdlg);
 }
 
 
@@ -377,7 +324,7 @@ genre_dialog_key_press (GtkWidget   *widget,
                         gpointer     user_data)
 {
   if (event->keyval == GDK_Escape)
-    gtk_widget_destroy (widget);
+    gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_NONE);
   return FALSE;
 }
 
@@ -389,55 +336,52 @@ genre_dialog (void)
   gint i, j;
   GType types[258];
   GtkTreeViewColumn *column;
-  GtkWidget *dialog, *button5, *scroll;
-  GtkWidget *hbox, *vbox;
+  GtkWidget *scroll, *hbox, *vbox;
+  GenreDialog gnrdlg;
 
   /* ja:メインウインドウ */
-  dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Genre"));
-  gtk_window_set_default_size (GTK_WINDOW (dialog),
+  gnrdlg.dialog = gtk_dialog_new_with_buttons
+                            (_("Genre"),
+                            GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_STOCK_OK, GTK_RESPONSE_OK,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            NULL);
+  gtk_window_set_default_size (GTK_WINDOW (gnrdlg.dialog),
                     gdk_screen_width () * 2 / 3, gdk_screen_height () * 2 / 3);
-  g_signal_connect_after (G_OBJECT (dialog), "key-press-event",
+  g_signal_connect_after (G_OBJECT (gnrdlg.dialog), "key-press-event",
                                     G_CALLBACK (genre_dialog_key_press), NULL);
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
   /* ja:ボタン */
-  button0 = gtk_button_new_with_mnemonic (_("_New"));
-  button1 = gtk_button_new_with_mnemonic (_("De_lete"));
-  button2 = gtk_button_new_with_mnemonic (_("_Up"));
-  button3 = gtk_button_new_with_mnemonic (_("_Down"));
-  button4 = gtk_button_new_with_label (_("OK"));
-  button5 = gtk_button_new_with_label (_("Cancel"));
-  g_signal_connect (G_OBJECT (button0), "clicked",
-                            G_CALLBACK (genre_dialog_clicked_new), NULL);
-  g_signal_connect (G_OBJECT (button1), "clicked",
-                            G_CALLBACK (genre_dialog_clicked_delete), NULL);
-  g_signal_connect (G_OBJECT (button2), "clicked",
-                            G_CALLBACK (genre_dialog_clicked_up), NULL);
-  g_signal_connect (G_OBJECT (button3), "clicked",
-                            G_CALLBACK (genre_dialog_clicked_down), NULL);
-  g_signal_connect (G_OBJECT (button4), "clicked",
-                            G_CALLBACK (genre_dialog_clicked_ok), dialog);
-  g_signal_connect_swapped (G_OBJECT (button5), "clicked",
-                        G_CALLBACK (gtk_widget_destroy), G_OBJECT (dialog));
-  GTK_WIDGET_SET_FLAGS (button0, GTK_CAN_DEFAULT);
-  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
+  gnrdlg.button0 = gtk_button_new_from_stock (GTK_STOCK_NEW);
+  gnrdlg.button1 = gtk_button_new_from_stock (GTK_STOCK_DELETE);
+  gnrdlg.button2 = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
+  gnrdlg.button3 = gtk_button_new_from_stock (GTK_STOCK_GO_DOWN);
+  g_signal_connect (G_OBJECT (gnrdlg.button0), "clicked",
+                            G_CALLBACK (genre_dialog_clicked_new), &gnrdlg);
+  g_signal_connect (G_OBJECT (gnrdlg.button1), "clicked",
+                            G_CALLBACK (genre_dialog_clicked_delete), &gnrdlg);
+  g_signal_connect (G_OBJECT (gnrdlg.button2), "clicked",
+                            G_CALLBACK (genre_dialog_clicked_up), &gnrdlg);
+  g_signal_connect (G_OBJECT (gnrdlg.button3), "clicked",
+                            G_CALLBACK (genre_dialog_clicked_down), &gnrdlg);
+  GTK_WIDGET_SET_FLAGS (gnrdlg.button0, GTK_CAN_DEFAULT);
+  GTK_WIDGET_SET_FLAGS (gnrdlg.button1, GTK_CAN_DEFAULT);
+  GTK_WIDGET_SET_FLAGS (gnrdlg.button2, GTK_CAN_DEFAULT);
+  GTK_WIDGET_SET_FLAGS (gnrdlg.button3, GTK_CAN_DEFAULT);
   /* ja:リストボックス */
   for (i = 0; i < 256; i++)
     types[i] = G_TYPE_STRING;
   types[256] = G_TYPE_INT;
   types[257] = G_TYPE_BOOLEAN;
-  store = gtk_list_store_newv (258, types);
-  tview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
-  g_signal_connect (G_OBJECT (tview), "cursor-changed",
-                            G_CALLBACK (genre_dialog_cursor_changed), NULL);
-  g_signal_connect (G_OBJECT (tview), "destroy",
-                                    G_CALLBACK (genre_dialog_destroy), NULL);
+  gnrdlg.store = gtk_list_store_newv (258, types);
+  gnrdlg.tview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (gnrdlg.store));
+  g_signal_connect (G_OBJECT (gnrdlg.tview), "cursor-changed",
+                            G_CALLBACK (genre_dialog_cursor_changed), &gnrdlg);
   /* ja:コラム */
   column = gtk_tree_view_column_new_with_attributes (NULL,
                             gtk_cell_renderer_text_new (), "text", 256, NULL);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tview), column);
-  g_object_set_data (G_OBJECT (column), "user_data", (gpointer)-1);
+  gtk_tree_view_append_column (GTK_TREE_VIEW (gnrdlg.tview), column);
+  g_object_set_data (G_OBJECT (column), "user_data", GINT_TO_POINTER (-1));
   for (i = 0; i < genres; i++)
     {
       gchar *title;
@@ -445,29 +389,29 @@ genre_dialog (void)
 
       renderer = gtk_cell_renderer_text_new ();
       g_signal_connect (renderer, "edited",
-                                    G_CALLBACK (genre_dialog_edited), NULL);
-      g_object_set_data (G_OBJECT (renderer), "user_data", (gpointer)i);
+                                    G_CALLBACK (genre_dialog_edited), &gnrdlg);
+      g_object_set_data (G_OBJECT (renderer), "user_data",
+                                                        GINT_TO_POINTER (i));
       title = g_strdup_printf ("%d", i);
       column = gtk_tree_view_column_new_with_attributes (title,
                                 renderer, "text", i, "editable", 257, NULL);
       g_free (title);
       gtk_tree_view_column_set_resizable (column, TRUE);
-      gtk_tree_view_append_column (GTK_TREE_VIEW (tview), column);
-      g_object_set_data (G_OBJECT (column), "user_data", (gpointer)i);
+      gtk_tree_view_append_column (GTK_TREE_VIEW (gnrdlg.tview), column);
+      g_object_set_data (G_OBJECT (column), "user_data", GINT_TO_POINTER (i));
     }
   for (i = 0; i < 256; i++)
     {
       GtkTreeIter iter;
 
-      gtk_list_store_append (store, &iter);
-      gtk_list_store_set (store, &iter, 256, i, 257, TRUE, -1);
+      gtk_list_store_append (gnrdlg.store, &iter);
+      gtk_list_store_set (gnrdlg.store, &iter, 256, i, 257, TRUE, -1);
       for (j = 0; j < genres; j++)
-        gtk_list_store_set (store, &iter,
-                        j, genre[j][i] ? g_strdup (genre[j][i]) : NULL, -1);
+        gtk_list_store_set (gnrdlg.store, &iter, j, genre[j][i], -1);
     }
   /* ja:スクロールウインドウ */
   scroll = gtk_scrolled_window_new (NULL, NULL);
-  gtk_container_add (GTK_CONTAINER (scroll), tview);
+  gtk_container_add (GTK_CONTAINER (scroll), gnrdlg.tview);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   /* ja:フレームとボックス */
@@ -475,25 +419,59 @@ genre_dialog (void)
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
   hbox = gtk_hbox_new (FALSE, SPACING);
-  gtk_box_pack_start (GTK_BOX (hbox), button0, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), button2, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox), button3, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), button5, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), button4, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), gnrdlg.button0, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), gnrdlg.button1, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), gnrdlg.button2, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), gnrdlg.button3, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (dialog), vbox);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (gnrdlg.dialog)->vbox), vbox);
 
   /* ja:表示 */
-  gtk_widget_set_sensitive (button0, FALSE);
-  gtk_widget_set_sensitive (button1, FALSE);
-  gtk_widget_set_sensitive (button2, FALSE);
-  gtk_widget_set_sensitive (button3, FALSE);
-  gtk_widget_set_sensitive (button4, FALSE);
-  gtk_widget_grab_focus (button5);
+  gtk_widget_set_sensitive (gnrdlg.button0, FALSE);
+  gtk_widget_set_sensitive (gnrdlg.button1, FALSE);
+  gtk_widget_set_sensitive (gnrdlg.button2, FALSE);
+  gtk_widget_set_sensitive (gnrdlg.button3, FALSE);
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (gnrdlg.dialog),
+                                                    GTK_RESPONSE_OK, FALSE);
+  gtk_widget_show_all (vbox);
 
-  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
-  gtk_grab_add (dialog);
-  gtk_widget_show_all (dialog);
-  gtk_main ();
+  if (gtk_dialog_run (GTK_DIALOG (gnrdlg.dialog)) == GTK_RESPONSE_OK)
+    {
+      gint k;
+      GList *glist;
+
+      for (i = 0; i < genres; i++)
+        for (j = 0; j < 256; j++)
+          g_free (genre[i][j]);
+      g_memset (genre, 0, sizeof (gchar *) * genres * 256);
+      glist = gtk_tree_view_get_columns (GTK_TREE_VIEW (gnrdlg.tview));
+      genres = g_list_length (glist) - 1;
+      g_list_free (glist);
+      for (i = 0; i < 256; i++)
+        {
+          GtkTreeIter iter;
+
+          gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (gnrdlg.store), &iter,
+                                                                    NULL, i);
+          for (j = 0; j < genres; j++)
+            gtk_tree_model_get (GTK_TREE_MODEL (gnrdlg.store), &iter,
+                                                        j, &genre[j][i], -1);
+        }
+      for (i = 0; i < genres; i++)
+        {
+          for (j = 0; j < 256; j++)
+            if (genre[i][j])
+              break;
+          if (j >= 256)
+            {
+              genres--;
+              for (j = i; j < genres; j++)
+                for (k = 0; k < 256; k++)
+                  genre[j][k] = genre[j + 1][k];
+              for (j = 0; j < 256; j++)
+                genre[genres][j] = NULL;
+            }
+        }
+    }
+  gtk_widget_destroy (gnrdlg.dialog);
 }
