@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include <gdk/gdkkeysyms.h>
 #include "reload.h"
@@ -26,39 +26,29 @@
 * ja:リロードダイアログ関数群                                                 *
 *                                                                             *
 ******************************************************************************/
-static gboolean reload_clicked_ok;
-static GtkWidget *dialog, *button0, *charset;
-
-
-/* ja:OKボタンの表示/非表示を判定 */
+/* ja:コンボボックスが変更された */
 static void
-reload_dialog_ok_sensitive (GtkWidget *widget,
-                            gpointer   user_data)
+reload_dialog_edited (GtkWidget *widget,
+                            GtkWidget *dialog)
 {
-  gtk_widget_set_sensitive (button0,
-                            orz_charset_is_valid (ORZ_CHARSET (charset)));
-}
+  GtkWidget *charset;
 
-
-/* ja:OKボタンが押された */
-static void
-reload_dialog_clicked (GtkWidget   *widget,
-                       MmaidWindow *mmaid)
-{
-  g_free (mmaid->charset);
-  mmaid->charset = orz_charset_get_charset (ORZ_CHARSET (charset));
-  gtk_widget_destroy (dialog);
-  reload_clicked_ok = TRUE;
+  charset = g_object_get_data (G_OBJECT (dialog), "user_data");
+  gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT,
+                                orz_charset_is_valid (ORZ_CHARSET (charset)));
 }
 
 
 /* ja:コンボボックスでリターンが押された */
 static void
 reload_dialog_activate (GtkWidget *widget,
-                        gpointer   user_data)
+                        GtkWidget *dialog)
 {
-  if (GTK_WIDGET_IS_SENSITIVE (button0))
-    reload_dialog_clicked (widget, user_data);
+  GtkWidget *charset;
+
+  charset = g_object_get_data (G_OBJECT (dialog), "user_data");
+  if (orz_charset_is_valid (ORZ_CHARSET (charset)))
+    gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 }
 
 
@@ -69,7 +59,7 @@ reload_dialog_key_press (GtkWidget   *widget,
                          gpointer     user_data)
 {
   if (event->keyval == GDK_Escape)
-    gtk_widget_destroy (widget);
+    gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_NONE);
   return FALSE;
 }
 
@@ -81,61 +71,55 @@ gboolean
 reload_dialog (MmaidWindow *mmaid,
                const gchar *text)
 {
-  GtkWidget *button1, *hbox, *vbox;
+  gboolean result;
+  GtkWidget *dialog, *charset, *vbox;
 
   /* ja:メインウインドウ */
-  dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Reload"));
+  dialog = gtk_dialog_new_with_buttons
+                            (_("Reload"),
+                            GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            NULL);
   g_signal_connect_after (G_OBJECT (dialog), "key-press-event",
                                 G_CALLBACK (reload_dialog_key_press), NULL);
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
   /* ja:ボタン */
   if (text)
-    {
-      button0 = gtk_button_new_with_mnemonic (_("_Yes"));
-      button1 = gtk_button_new_with_mnemonic (_("_No"));
-    }
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                            GTK_STOCK_YES, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_NO, GTK_RESPONSE_REJECT,
+                            NULL);
   else
-    {
-      button0 = gtk_button_new_with_label (_("OK"));
-      button1 = gtk_button_new_with_label (_("Cancel"));
-    }
-  g_signal_connect (G_OBJECT (button0), "clicked",
-                        G_CALLBACK (reload_dialog_clicked), mmaid);
-  g_signal_connect_swapped (G_OBJECT (button1), "clicked",
-                        G_CALLBACK (gtk_widget_destroy), G_OBJECT (dialog));
-  GTK_WIDGET_SET_FLAGS (button0, GTK_CAN_DEFAULT);
-  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
+    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                            GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+                            NULL);
   /* ja:文字符号化方式 */
   charset = orz_charset_new ();
   orz_charset_add_candidate (ORZ_CHARSET (charset), charset_list);
   orz_charset_set_charset (ORZ_CHARSET (charset), mmaid->charset);
   g_signal_connect (ORZ_CHARSET (charset), "activate",
-                                G_CALLBACK (reload_dialog_activate), mmaid);
+                                G_CALLBACK (reload_dialog_activate), dialog);
   g_signal_connect (ORZ_CHARSET (charset), "edited",
-                                G_CALLBACK (reload_dialog_ok_sensitive), NULL);
+                                G_CALLBACK (reload_dialog_edited), dialog);
+  g_object_set_data (G_OBJECT (dialog), "user_data", charset);
   /* ja:フレームとボックス */
-  hbox = gtk_hbox_new (FALSE, SPACING);
-  gtk_box_pack_end (GTK_BOX (hbox), button1, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (hbox), button0, FALSE, FALSE, 0);
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
   if (text)
     gtk_box_pack_start (GTK_BOX (vbox), gtk_label_new (text), FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), charset, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (dialog), vbox);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
 
   /* ja:表示 */
-  gtk_widget_grab_focus (text ? button1 : button0);
+  gtk_widget_show_all (vbox);
 
-  reload_clicked_ok = FALSE;
+  result = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT;
+  if (result)
+    {
+      g_free (mmaid->charset);
+      mmaid->charset = orz_charset_get_charset (ORZ_CHARSET (charset));
+    }
+  gtk_widget_destroy (dialog);
 
-  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
-  gtk_grab_add (dialog);
-  gtk_widget_show_all (dialog);
-  gtk_main ();
-
-  return reload_clicked_ok;
+  return result;
 }
