@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include "command.h"
 #include "toolbar.h"
@@ -26,7 +26,7 @@
 *                                                                             *
 ******************************************************************************/
 /* XPM */
-static const gchar *tb_new_xpm[] = {
+const static gchar *tb_new_xpm[] = {
 "16 16 4 1",
 "   c None",
 ".  c #000000",
@@ -51,7 +51,7 @@ static const gchar *tb_new_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_open_xpm[] = {
+const static gchar *tb_open_xpm[] = {
 "16 16 5 1",
 "   c None",
 ".  c #000000",
@@ -77,7 +77,7 @@ static const gchar *tb_open_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_save_xpm[] = {
+const static gchar *tb_save_xpm[] = {
 "16 16 5 1",
 "  c None",
 ". c #000000",
@@ -103,7 +103,7 @@ static const gchar *tb_save_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_cut_xpm[] = {
+const static gchar *tb_cut_xpm[] = {
 "16 16 5 1",
 "   c None",
 ".  c #000000",
@@ -129,7 +129,7 @@ static const gchar *tb_cut_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_copy_xpm[] = {
+const static gchar *tb_copy_xpm[] = {
 "16 16 4 1",
 "   c None",
 ".  c #000000",
@@ -154,7 +154,7 @@ static const gchar *tb_copy_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_paste_xpm[] = {
+const static gchar *tb_paste_xpm[] = {
 "16 16 5 1",
 "   c None",
 ".  c #000000",
@@ -180,7 +180,7 @@ static const gchar *tb_paste_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_delete_xpm[] = {
+const static gchar *tb_delete_xpm[] = {
 "16 16 2 1",
 "   c None",
 ",  c #0000FF",
@@ -203,7 +203,7 @@ static const gchar *tb_delete_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_root_xpm[] = {
+const static gchar *tb_root_xpm[] = {
 "16 16 5 1",
 "   c None",
 ".  c #000000",
@@ -229,7 +229,7 @@ static const gchar *tb_root_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_refresh_xpm[] = {
+const static gchar *tb_refresh_xpm[] = {
 "16 16 4 1",
 "   c None",
 ".  c #000000",
@@ -254,7 +254,7 @@ static const gchar *tb_refresh_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_add_xpm[] = {
+const static gchar *tb_add_xpm[] = {
 "16 16 5 1",
 "   c None",
 ".  c #000000",
@@ -280,7 +280,7 @@ static const gchar *tb_add_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_edit_xpm[] = {
+const static gchar *tb_edit_xpm[] = {
 "16 16 7 1",
 "   c None",
 ".  c #000000",
@@ -308,7 +308,7 @@ static const gchar *tb_edit_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_search_xpm[] = {
+const static gchar *tb_search_xpm[] = {
 "16 16 6 1",
 " 	c None",
 ".	c #000000",
@@ -334,7 +334,7 @@ static const gchar *tb_search_xpm[] = {
 "                "};
 
 /* XPM */
-static const gchar *tb_up_xpm[] = {
+const static gchar *tb_up_xpm[] = {
 "16 16 2 1",
 "   c None",
 "-  c #FF0000",
@@ -357,7 +357,7 @@ static const gchar *tb_up_xpm[] = {
 
 
 /* XPM */
-static const gchar *tb_down_xpm[] = {
+const static gchar *tb_down_xpm[] = {
 "16 16 2 1",
 "   c None",
 ",  c #0000FF",
