@@ -14,11 +14,11 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 #include <gdk/gdkkeysyms.h>
 #include "version.h"
-#include "misc/license.h"
+#include "misc/gnugpl.h"
 
 
 /******************************************************************************
@@ -33,7 +33,7 @@ version_dialog_key_press (GtkWidget   *widget,
                           gpointer     user_data)
 {
   if (event->keyval==GDK_Escape)
-    gtk_widget_destroy(widget);
+    gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_NONE);
   return FALSE;
 }
 
@@ -45,7 +45,7 @@ version_dialog (void)
   GdkPixbuf *pixbuf;
   GtkTextBuffer *buffer;
   GtkTextIter where;
-  GtkWidget *dialog, *button, *notebook, *hbox0, *hbox1, *vbox;
+  GtkWidget *dialog, *notebook, *hbox0, *hbox1, *vbox;
   GtkWidget *text, *scroll;
 const static gchar *authors =
 "Main Program :\n"
@@ -59,7 +59,7 @@ const static gchar *authors =
 "Special Thanks :\n"
 "    kyun2\n"
 "    And all Melody maid users";
-static const gchar *mmaid_xpm[] = {
+const static gchar *mmaid_xpm[] = {
 "32 32 10 1",
 "   c None",
 ".  c #000000",
@@ -105,14 +105,17 @@ static const gchar *mmaid_xpm[] = {
 "   =========================    "};
 
   /* ja:メインウインドウ */
-  dialog = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (dialog), _("About Melody maid"));
+  dialog = gtk_dialog_new_with_buttons
+                            (_("About Melody maid"),
+                            GTK_WINDOW (window),
+                            GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                            GTK_STOCK_OK, GTK_RESPONSE_OK,
+                            NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   gtk_window_set_default_size (GTK_WINDOW (dialog),
                     gdk_screen_width () * 2 / 3, gdk_screen_height () * 2 / 3);
   g_signal_connect (G_OBJECT (dialog), "key-press-event",
                                 G_CALLBACK (version_dialog_key_press), NULL);
-  g_signal_connect (G_OBJECT (dialog), "destroy",
-                                            G_CALLBACK (gtk_main_quit), NULL);
   /* ja:ノートブック */
   notebook = gtk_notebook_new ();
   gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
@@ -122,7 +125,7 @@ static const gchar *mmaid_xpm[] = {
   /* ja:テキスト */
   text = gtk_text_view_new ();
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text));
-  gtk_text_buffer_set_text (buffer, gnu_gpl, -1);
+  gtk_text_buffer_set_text (buffer, gnugpl, -1);
   gtk_text_buffer_get_start_iter (buffer, &where);
   gtk_text_buffer_place_cursor (buffer, &where);
   gtk_text_view_set_editable (GTK_TEXT_VIEW (text), FALSE);
@@ -152,12 +155,6 @@ static const gchar *mmaid_xpm[] = {
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), scroll,
                                                 gtk_label_new (_("Authors")));
 
-  /* ja:ボタン */
-  button = gtk_button_new_with_label (_("OK"));
-  g_signal_connect_swapped (G_OBJECT (button), "clicked",
-                        G_CALLBACK (gtk_widget_destroy), G_OBJECT (dialog));
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-
   /* ja:フレームとボックス */
   vbox = gtk_vbox_new (FALSE, SPACING);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), SPACING);
@@ -173,16 +170,11 @@ static const gchar *mmaid_xpm[] = {
   gtk_box_pack_start (GTK_BOX (hbox1), hbox0, TRUE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX(vbox), hbox1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 0);
-  hbox0 = gtk_hbox_new (FALSE, SPACING);
-  gtk_box_pack_end (GTK_BOX (hbox0), button, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox0, FALSE, FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (dialog), vbox);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
 
   /* ja:表示 */
-  gtk_widget_grab_focus (button);
+  gtk_widget_show_all (vbox);
 
-  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
-  gtk_grab_add (dialog);
-  gtk_widget_show_all (dialog);
-  gtk_main ();
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
 }
